shader = [[
  
  struct Triangle {
    vec2 pos1;
    vec2 pos2;
    vec2 pos3;
  };
  
  extern Triangle triangles[1000];
  extern int num_triangles;
  
  vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords){
    vec4 pixel = Texel(texture, texture_coords);
    
    for (int i = 0; i < num_triangles; i++) {
      Triangle triangle = triangles[i];
      vec2 pos1 = triangle.pos1;
      vec2 pos2 = triangle.pos2;
      vec2 pos3 = triangle.pos3;
      float px = screen_coords.x;
      float py = screen_coords.y;
      float areaOrig = abs((pos2.x-pos1.x)*(pos3.y-pos1.y) - (pos3.x-pos1.x)*(pos2.y-pos1.y));
      float area1 = abs((pos1.x-px)*(pos2.y-py) - (pos2.x-px)*(pos1.y-py));
      float area2 = abs((pos2.x-px)*(pos3.y-py) - (pos3.x-px)*(pos2.y-py));
      float area3 = abs((pos3.x-px)*(pos1.y-py) - (pos1.x-px)*(pos3.y-py));
      
      float factor = 0.0001;
      if ((area1 + area2 + area3) / areaOrig > 1 - factor && (area1 + area2 + area3) / areaOrig < 1 + factor) {
        return pixel * color;
        break;
      }
      
    }
    
    return pixel * color * vec4(0.2, 0.2, 0.2, 1);
    
    
    
  }
  
]]







function love.load()
  
  floor = love.graphics.newImage("floor.png")
  wall = love.graphics.newImage("wall.png")
  human = love.graphics.newImage("human.png")
  FOVshader = love.graphics.newShader(shader)
  
  ogsize = 32
  viewsize = 8
  run_shader = true
  screen = {}
  collisions = {}
  map = {}
  rays = {}
  triangles = {}
  screen.width, screen.height = love.window.getMode()
  size = math.floor(screen.height / ((viewsize * 2) + 1))
  player = {x = 100, y = 100, drawx = 3200, drawy = 3200, speed = 1, timer = {max = 100, speed = 800, count = -1}}
  vision = {x = 0, y = 0}
  camera = {dx = 0, dy = 0}
  
  
  for x = 1, 200, 1 do
    map[x] = {}
    for y = 1, 200, 1 do 
      map[x][y] = {solid = false}
    end
  end
  for x = 105, 115,1 do
    map[x][100].solid = true
  end
  for x = 105, 115,1 do
    map[x][102].solid = true
  end
  map[108][97].solid = true
  
  player.drawx = player.x * size
  player.drawy = player.y * size
  
end



function distance(x1, y1, x2, y2)
  if not(x1) or not(y1) or not(x2) or not(y2) then return math.huge end
  return math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
end

function intersection(x1,y1,x2,y2, x3,y3,x4,y4)
  
  uA = ((x4-x3)*(y1-y3) - (y4-y3)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1))
  uB = ((x2-x1)*(y1-y3) - (y2-y1)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1))
  
  if (uA >= 0 and uA <= 1 and uB >= 0 and uB <= 1) then
    
    intersectionX = x1 + (uA * (x2-x1));
    intersectionY = y1 + (uA * (y2-y1));
    return {x = intersectionX, y = intersectionY}
    
  end
  
end

function raycast(x1,y1,x2,y2)
  collision = {}
  for x = player.x - viewsize, player.x + viewsize, 1 do
    if map[x] then
      for y = player.y - viewsize, player.y + viewsize, 1 do
        if map[x][y] and map[x][y].solid then
          tilex = x * size
          tiley = y * size
          left= intersection(x1, y1, x2, y2, tilex, tiley, tilex, (tiley) + size)
          right = intersection(x1, y1, x2, y2, (tilex) + size, tiley, (tilex) + size, (tiley) + size)
          top = intersection(x1, y1, x2, y2, tilex, tiley, (tilex) + size, tiley)
          bottom = intersection(x1, y1, x2, y2, tilex, (tiley) + size, (tilex) + size, (tiley) + size)
          
          collision[#collision + 1] = left
          collision[#collision + 1] = right
          collision[#collision + 1] = top
          collision[#collision + 1] = bottom
        end
      end
    end
  end
  
  
  local origin = {x = camera.dx, y = camera.dy}
  local viewport = (viewsize * size * 2) + size
  left = intersection(x1, y1, x2, y2, origin.x, origin.y, origin.x, origin.y + viewport)
  right = intersection(x1, y1, x2, y2, origin.x + viewport, origin.y, origin.x + viewport, origin.y + viewport)
  top = intersection(x1, y1, x2, y2, origin.x, origin.y, origin.x + viewport, origin.y)
  bottom = intersection(x1, y1, x2, y2, origin.x, origin.y + viewport, origin.x + viewport, origin.y + viewport)
  if left then
    collision[#collision + 1] = left
    collision[#collision].wall = true
  end
  if right then
    collision[#collision + 1] = right
    collision[#collision].wall = true
  end
  if top then
    collision[#collision + 1] = top
    collision[#collision].wall = true
  end
  if bottom then
    collision[#collision + 1] = bottom
    collision[#collision].wall = true
  end
  
  closest = 1
  for i, obj in ipairs(collision) do
    if distance(x1,y1,obj.x, obj.y) < distance(x1,y1,collision[closest].x, collision[closest].y) then
      closest = i
    end
  end
  
  collision = collision[closest]
  
  
  return collision
  
end



function love.update(dt)
  size = math.floor(screen.height / ((viewsize * 2) + 1))
  proportion = size / ogsize
  
  player.timer.count = player.timer.count - (player.timer.speed * dt)
  if player.timer.count < 0 then
    if love.keyboard.isDown('w') then
      player.y = player.y - player.speed
      player.timer.count = player.timer.max
    elseif love.keyboard.isDown('s') then
      player.y = player.y + player.speed
      player.timer.count = player.timer.max
    elseif love.keyboard.isDown('a') then
      player.x = player.x - player.speed
      player.timer.count = player.timer.max
    elseif love.keyboard.isDown('d') then
      player.x = player.x + player.speed
      player.timer.count = player.timer.max
    end
  end
  
  player.drawx = player.x * size
  player.drawy = player.y * size
  player.centerx = player.drawx + size / 2
  player.centery = player.drawy + size / 2
  camera.dx = player.drawx - (viewsize * size)
  camera.dy = player.drawy - (viewsize * size)
  vision.x, vision.y = love.mouse.getPosition()
  vision.x = vision.x + camera.dx
  vision.y = vision.y + camera.dy
  vision.angle = math.atan2(vision.y - player.centery, vision.x - player.centerx)
  vision.x = player.centerx + (math.cos(vision.angle) * distance(player.centerx, player.centery, camera.dx, camera.dy))
  vision.y = player.centery + (math.sin(vision.angle) * distance(player.centerx, player.centery, camera.dx, camera.dy))
  camera.dx = player.drawx - (viewsize * size)
  camera.dy = player.drawy - (viewsize * size)
  
  
  local viewport = (viewsize * size * 2) + size
  rays = {}
  rays[#rays + 1] = {x = vision.x, y = vision.y}
  rays[#rays + 1] = {x = camera.dx, y = camera.dy}
  rays[#rays + 1] = {x = viewport + camera.dx, y = camera.dy}
  rays[#rays + 1] = {x = camera.dx, y = viewport + camera.dy}
  rays[#rays + 1] = {x = viewport + camera.dx, y = viewport + camera.dy}

  
  for x = player.x - viewsize, player.x + viewsize, 1 do
    if map[x] then
      for y = player.y - viewsize, player.y + viewsize, 1 do
        if map[x][y] then
          obj = map[x][y]
          if obj.solid then
            local radius = distance(player.centerx, player.centery, camera.dx, camera.dy)
            local angle = math.atan2((y * size) - player.centery, (x * size) - player.centerx)
            posx = player.centerx + (math.cos(angle) * radius)
            posy = player.centery + (math.sin(angle) * radius)
            rays[#rays + 1] = {x = posx, y = posy}
            
            angle = math.atan2((y * size) - player.centery, ((x * size) + size) - player.centerx)
            posx = player.centerx + (math.cos(angle) * radius)
            posy = player.centery + (math.sin(angle) * radius)
            rays[#rays + 1] = {x = posx, y = posy}
            
            angle = math.atan2(((y * size) + size) - player.centery, (x * size) - player.centerx)
            posx = player.centerx + (math.cos(angle) * radius)
            posy = player.centery + (math.sin(angle) * radius)
            rays[#rays + 1] = {x = posx, y = posy}
            
            angle = math.atan2(((y * size) + size) - player.centery, ((x * size) + size) - player.centerx)
            posx = player.centerx + (math.cos(angle) * radius)
            posy = player.centery + (math.sin(angle) * radius)
            rays[#rays + 1] = {x = posx, y = posy}
          end
        end
      end
    end
  end
  
  collisions = {}
  for i, obj in ipairs(rays) do
    cast = raycast(player.centerx, player.centery, obj.x, obj.y)
    if cast and not(obj.cluster) then
      local angle = math.atan2(obj.y - player.centery, obj.x - player.centerx)
      local radius = distance(player.centerx, player.centery, camera.dx, camera.dy)
      factor = 0.001
      rays[#rays + 1] = {x = player.centerx + (math.cos(angle + factor) * radius), y = player.centery + (math.sin(angle + factor) * radius), cluster = true}
      rays[#rays + 1] = {x = player.centerx + (math.cos(angle - factor) * radius), y = player.centery + (math.sin(angle - factor) * radius), cluster = true}
    end
      
    obj.collision = cast
    collisions[#collisions + 1] = cast
  end
  
  sortedcollisions = {}
  while #collisions ~= 0 do
    closest = 1
    for i, obj in ipairs(collisions) do
      angle1 = math.atan2(collisions[closest].y - player.centery, collisions[closest].x - player.centerx)
      angle2 = math.atan2(obj.y - player.centery, obj.x - player.centerx)
      
      if angle2 < angle1 then
        closest = i
      end
    end
    
    sortedcollisions[#sortedcollisions + 1] = collisions[closest]
    table.remove(collisions, closest)
  end
  
  collisions = sortedcollisions
  
  triangles = {}
  for i, obj in ipairs(collisions) do
    if collisions[i + 1] then
      triangles[#triangles + 1] = {player.centerx, player.centery, obj.x, obj.y, collisions[i + 1].x, collisions[i + 1].y}
    else
      triangles[#triangles + 1] = {player.centerx, player.centery, obj.x, obj.y, collisions[1].x, collisions[1].y}
    end
  end
  
  
  
  
  
  
  
  
  
end


function love.mousepressed(x,y,button)
  
  if button == 1 then
    xrounded = math.floor((x + camera.dx) / size)
    yrounded = math.floor((y + camera.dy) / size)
    map[xrounded][yrounded].solid = not(map[xrounded][yrounded].solid)
  end
  
  if button == 2 then
    run_shader = not(run_shader)
  end
  
end

function love.draw()
  
  love.graphics.translate(-camera.dx, -camera.dy)
  
  if run_shader then
    love.graphics.setShader(FOVshader)
  end
  for i, obj in ipairs(triangles) do
    FOVshader:send("triangles[" .. i - 1 .. "].pos1", {obj[1] - camera.dx, obj[2] - camera.dy})
    FOVshader:send("triangles[" .. i - 1 .. "].pos2", {obj[3] - camera.dx, obj[4] - camera.dy})
    FOVshader:send("triangles[" .. i - 1 .. "].pos3", {obj[5] - camera.dx, obj[6] - camera.dy})
  end
  FOVshader:send("num_triangles", #triangles)
  
  
  for x = player.x - viewsize, player.x + viewsize, 1 do
    if map[x] then
      for y = player.y - viewsize, player.y + viewsize, 1 do
        if map[x][y] then
          obj = map[x][y]
          if obj.solid then
            if run_shader then
              love.graphics.setShader();
            end
            love.graphics.draw(wall, (x * size), (y * size),0, proportion)
          else
            if run_shader then
              love.graphics.setShader(FOVshader)
            end
            love.graphics.draw(floor, (x * size), (y * size),0, proportion)
          end
        end
      end
    end
  end
  
  for i, obj in ipairs(rays) do
    --love.graphics.line(player.centerx, player.centery, obj.x, obj.y)
    if obj.collision then
      --love.graphics.line(player.centerx, player.centery, obj.collision.x, obj.collision.y)
      love.graphics.circle("fill", obj.collision.x, obj.collision.y, 4)
    end
  end
  
  for i, obj in ipairs(collisions) do
    --love.graphics.circle("fill", obj.x, obj.y, 4)
    --love.graphics.setColor(0,0,0)
    --love.graphics.rectangle("fill", obj.x, obj.y, 15, 15)
    --love.graphics.setColor(1,1,1)
    --love.graphics.print(i, obj.x, obj.y)
  end
  
  for i, obj in ipairs(triangles) do
    --love.graphics.polygon("line", obj)
    --love.graphics.line(obj[1], obj[2], obj[3], obj[4])
    --love.graphics.line(obj[3], obj[4], obj[5], obj[6])
    --love.graphics.line(obj[5], obj[6], obj[1], obj[2])
  end
  
  love.graphics.draw(human, player.drawx, player.drawy, 0, proportion)
  love.graphics.circle("line", player.centerx, player.centery, distance(player.centerx, player.centery, camera.dx, camera.dy), 100)
  love.graphics.line(player.drawx + size / 2, player.drawy + size / 2, vision.x, vision.y)
  
  love.graphics.translate(camera.dx,camera.dy)
  love.graphics.setShader()
  love.graphics.print("FPS = " .. love.timer.getFPS(), 15, 15)
  
end











